import del from 'rollup-plugin-delete';
import { terser } from 'rollup-plugin-terser';
import { nodeResolve } from '@rollup/plugin-node-resolve';
import { visualizer } from 'rollup-plugin-visualizer';
import replace from '@rollup/plugin-replace';
import { babel } from '@rollup/plugin-babel';
import html from '@rollup/plugin-html';
import copy from 'rollup-plugin-copy';
import { readFileSync } from 'fs';

// If the build is happening in watch mode, we're in dev otherwis the build is
// for prod. TBH this isn't great. Really should have a way to build dev without
// having to watch.
const env = process.env.ROLLUP_WATCH ? 'development' : 'production';

export default {
  input: 'src/main.js',
  output: {
    dir: 'dist/',
    entryFileNames: '[name].[hash].js',
    sourcemap: env === 'development',
    format: 'iife',
    name: 'wordle.bundle',
  },
  plugins: [
    watch('src/index.html'),
    watch('src/reset.html'),
    del({ targets: 'dist/*', verbose: true }),
    nodeResolve(),
    replace({
      'process.env.ENV': JSON.stringify(env),
      preventAssignment: true,
    }),
    babel({
      babelHelpers: 'bundled',
      presets: ['@babel/preset-env'],
    }),
    terser({
      compress: {
        drop_console: env === 'production',
      },
    }),
    html({
      template: ({ attributes, bundle, files, publicPath, title }) => {
        // Not really sure about a bunch of this
        const [fileName] = Object.keys(bundle);
        // main here refers to the name of the bundle which i think comes from the entrypoint
        const match = fileName.match(/^main.(?<hash>.*).js/);
        const { hash } = match.groups;
        const htmlTemplate = readFileSync('src/index.html').toString();
        let html = htmlTemplate;
        html = html.replace('${script}', fileName);
        html = html.replace('${hash}', hash);
        return html;
      },
    }),
    copy({
      targets: [
        { src: './assets/*', dest: './dist/images/' },
        { src: './manifest.json', dest: './dist/' },
        { src: './src/fonts/*', dest: './dist/fonts/' },
        { src: './src/reset.html', dest: './dist' },
      ],
    }),
    visualizer(),
  ],
};

// https://dev.to/lukap/creating-a-rollup-plugin-to-copy-and-watch-a-file-3hi2
function watch(fileIn) {
  return {
    name: 'watch',
    async buildStart() {
      this.addWatchFile(fileIn);
    },
  };
}
