// Script to log the dates of deploys, along with bundle and commit hashes

const fs = require('fs');

const now = new Date().toISOString();

const hashFileNameRegex = /^main.(?<hash>[a-z0-9]*).js$/;
const fileNames = fs.readdirSync('dist/');
let hash = '';

for (let i = 0; i < fileNames.length; i++) {
  const fileName = fileNames[i];
  const match = fileName.match(hashFileNameRegex);
  if (match && match.groups && match.groups.hash) {
    hash = match.groups.hash;
    break;
  }
}

if (hash === '') {
  console.log('LOG DEPLOY: ERROR - No hash name');
  return;
}

const commitHash = require('child_process').execSync('git rev-parse HEAD').toString().trim();

const content = `${now},${hash},${commitHash}\n`;
console.log(`LOG DEPLOY: Logging: ${content}`);

fs.appendFileSync('deploys.log', content);
