// A script to move words from the solution word list to the valid word list and
// vice-versa. It does not add or remove words.

// The new lists are written to /output. They need to be manually copied into
// the corresponding js files.

import { solutionWords } from '../src/words/solutionWords';
import { validWords } from '../src/words/validWords';
import { solutionToValid, validToSolution } from './newWords';
import fs from 'fs';

// https://stackoverflow.com/a/12646864/6381852
function shuffleArray(array) {
  for (let i = array.length - 1; i > 0; i--) {
    const j = Math.floor(Math.random() * (i + 1));
    [array[i], array[j]] = [array[j], array[i]];
  }
}

// Verify no duplicates in either
function containsDuplicates(array) {
  if (new Set(array).size === array.length) return false;

  const duplicates = array.filter((word, index) => array.indexOf(word) !== index);
  console.log(`Duplicates: ${duplicates.length}`);
  console.log(duplicates);

  return true;
}

console.log('Checking existing words for duplicates...');

if (containsDuplicates(solutionWords)) {
  console.log('Solution words contain duplicates, Aborting');
  process.exit();
}

if (containsDuplicates(validWords)) {
  console.log('Valid words contain duplicates, Aborting');
  process.exit();
}

console.log('No duplicates found, continuing...');

// Where possible, we don't want to disrupte the current wordlist

const solutionToValidLength = solutionToValid.length;
const validToSolutionLength = validToSolution.length;
const predictedDiff = Math.abs(validToSolutionLength - solutionToValidLength);
console.log(`Solution > Valid: ${solutionToValidLength}`);
console.log(`Valid > Solution: ${validToSolutionLength}`);

if (solutionToValidLength > validToSolutionLength) {
  console.log(`This would REDUCE the number of solution words by ${predictedDiff}`);
  console.log('ABORTING');
  process.exit();
} else if (validToSolutionLength > solutionToValidLength) {
  console.log(`This will INCREASE the number of solution words by ${predictedDiff}`);
} else {
  console.log('This will NOT CHANGE the number of solution words');
}

// Randomize the validToSolutionWords
shuffleArray(validToSolution);

const newSolutionWords = [...solutionWords];
let newValidWords = [...validWords];

const tempValidToSolution = [...validToSolution];

solutionToValid.forEach((word) => {
  const i = newSolutionWords.indexOf(word);
  const replacementWord = tempValidToSolution.pop();
  console.log(`Replacing ${word.toUpperCase()} with ${replacementWord.toUpperCase()}`);
  newSolutionWords[i] = replacementWord;
});

console.log(`${tempValidToSolution.length} words remain, adding to the end of solutionWords`);
newSolutionWords.push(...tempValidToSolution);

// TODO: remove validToSolutionWords from

console.log(`Moving ${solutionToValid.length} words to valid words`);
// Add all the solutionToValid words to validWords
newValidWords.push(...solutionToValid);
// filter out all the new solution words
newValidWords = newValidWords.filter((word) => !validToSolution.includes(word));
newValidWords.sort();

const totalWordDiff = Math.abs(
  validWords.length + solutionWords.length - (newValidWords.length + newSolutionWords.length),
);
console.log(`Previous valid words: ${validWords.length}`);
console.log(`New valid words: ${newValidWords.length}`);
console.log(`Previous solution words: ${solutionWords.length}`);
console.log(`New solution words: ${newSolutionWords.length}`);
console.log(`Predicted diff: ${predictedDiff}`);

console.log(`Actual Diff: ${newSolutionWords.length - solutionWords.length}`);
console.log(`Total Word diff: ${totalWordDiff}`);

if (containsDuplicates(newSolutionWords)) {
  console.log('newSolutionWords contains duplicates, aborting...');
  process.exit();
}

if (containsDuplicates(newValidWords)) {
  console.log('newValidWords contains duplicates, aborting...');
  process.exit();
}

if (containsDuplicates([...newValidWords, ...newSolutionWords])) {
  console.log('newValidWords and newSolutionWords contain duplicates of one another, aborting...');
  process.exit();
}

console.log('Writing new files...');

fs.writeFileSync('output/newSolutionWords.json', JSON.stringify(newSolutionWords, null, ' '));
fs.writeFileSync('output/newValidWords.json', JSON.stringify(newValidWords, null, ' '));
