const countdownTimerTemplate = document.createElement('template')

countdownTimerTemplate.innerHTML = `
  <div id="timer"></div>
`

const INTERVAL_MS = 200

const ONE_SECOND = 1000
const ONE_MINUTE = ONE_SECOND * 60
const ONE_HOUR = ONE_MINUTE * 60
const ONE_DAY = ONE_HOUR * 24

export class CountdownTimer extends HTMLElement {
  targetEpochMS
  intervalId
  $timer

  constructor() {
    super()
    this.attachShadow({ mode: 'open' })
    const targetDate = new Date()
    // Set to tomorrow
    targetDate.setDate(targetDate.getDate() + 1)
    // Set to midnight
    targetDate.setHours(0, 0, 0, 0)
    this.targetEpochMS = targetDate.getTime()
  }

  padDigit(digit) {
    return digit.toString().padStart(2, '0')
  }

  updateTimer() {
    const now = new Date().getTime()
    const msDiff = Math.floor(this.targetEpochMS - now)
    let time
    if (msDiff <= 0) {
      time = '00:00:00'
    } else {
      const hours = Math.floor((msDiff % ONE_DAY) / ONE_HOUR)
      const minutes = Math.floor((msDiff % ONE_HOUR) / ONE_MINUTE)
      const seconds = Math.floor((msDiff % ONE_MINUTE) / ONE_SECOND)

      time = `${this.padDigit(hours)}:${this.padDigit(
        minutes,
      )}:${this.padDigit(seconds)}`
    }
    this.$timer.textContent = time
  }

  connectedCallback() {
    this.shadowRoot.appendChild(
      countdownTimerTemplate.content.cloneNode(true),
    )
    this.$timer = this.shadowRoot.querySelector('#timer')
    this.intervalId = setInterval(() => {
      this.updateTimer()
    }, INTERVAL_MS)
  }

  disconnectedCallback() {
    clearInterval(this.intervalId)
  }
}

customElements.define('countdown-timer', CountdownTimer)
