const gameHelpTemplate = document.createElement('template');

gameHelpTemplate.innerHTML = `
  <style>
  .instructions {
    font-size: 14px;
    color: var(--color-tone-1)
  }

  .examples {
    border-bottom: 1px solid var(--color-tone-4);
    border-top: 1px solid var(--color-tone-4);
  }

  .example {
    margin-top: 24px;
    margin-bottom: 24px;
  }

  game-tile {
    width: 40px;
    height: 40px;
  }

  :host([page]) section {
    padding: 16px;
    padding-top: 0px;
  }

  </style>
  <section>
    <div class="instructions">
      <p>Guess the <strong>WORDLE</strong> in six tries.</p>
      <p>Each guess must be a valid five-letter word. Hit the enter button to submit.</p>
      <p>After each guess, the color of the tiles will change to show how close your guess was to the word.</p>
      <div class="examples">
        <p><strong>Examples</strong></p>
        <div class="example">
          <div class="row">
            <game-tile letter="w" evaluation="correct" reveal></game-tile>
            <game-tile letter="e"></game-tile>
            <game-tile letter="a"></game-tile>
            <game-tile letter="r"></game-tile>
            <game-tile letter="y"></game-tile>
          </div>
          <p>The letter <strong>W</strong> is in the word and in the correct spot.</p>
        </div>
        <div class="example">
          <div class="row">
            <game-tile letter="p"></game-tile>
            <game-tile letter="i" evaluation="present" reveal></game-tile>
            <game-tile letter="l"></game-tile>
            <game-tile letter="l"></game-tile>
            <game-tile letter="s"></game-tile>
          </div>
          <p>The letter <strong>I</strong> is in the word but in the wrong spot.</p>
        </div>
        <div class="example">
          <div class="row">
            <game-tile letter="v"></game-tile>
            <game-tile letter="a"></game-tile>
            <game-tile letter="g"></game-tile>
            <game-tile letter="u" evaluation="absent" reveal></game-tile>
            <game-tile letter="e"></game-tile>
          </div>
          <p>The letter <strong>U</strong> is not in the word in any spot.</p>
        </div>
      </div>
      <p><strong>A new WORDLE will be available each day!<strong></p>
    </div>
  </section>
`;

export class GameHelp extends HTMLElement {
  constructor() {
    super();
    this.attachShadow({ mode: 'open' });
  }

  connectedCallback() {
    this.shadowRoot.appendChild(gameHelpTemplate.content.cloneNode(true));
  }
}

customElements.define('game-help', GameHelp);
