import { MODAL_ZINDEX } from './game-app'

const modalTemplate = document.createElement('template')
modalTemplate.innerHTML = `
  <style>
    .overlay {
      display: none;
      position: absolute;
      width: 100%;
      height: 100%;
      top: 0;
      left: 0;
      justify-content: center;
      align-items: center;
      background-color: var(--opacity-50);
      z-index: ${MODAL_ZINDEX};
    }

    :host([open]) .overlay {
      display: flex;
    }

    .content {
      position: relative;
      border-radius: 8px;
      border: 1px solid var(--color-tone-6);
      background-color: var(--modal-content-bg);
      color: var(--color-tone-1);
      box-shadow: 0 4px 23px 0 rgba(0, 0, 0, 0.2);
      width: 90%;
      max-height: 90%;
      overflow-y: auto;
      animation: SlideIn 200ms;
      max-width: var(--game-max-width);
      padding: 16px;
      box-sizing: border-box;
    }

    .content.closing {
      animation: SlideOut 200ms;
    }

    .close-icon {
      width: 24px;
      height: 24px;
      position: absolute;
      top: 16px;
      right: 16px;
    }

    game-icon {
      position: fixed;
      user-select: none;
      cursor: pointer;
    }

    @keyframes SlideIn {
      0% {
        transform: translateY(30px);
        opacity: 0;
      }
      100% {
        transform: translateY(0px);
        opacity: 1;
      }
    }
    @keyframes SlideOut {
      0% {
        transform: translateY(0px);
        opacity: 1;
      }
      90% {
        opacity: 0;
      }
      100% {
        opacity: 0;
        transform: translateY(60px);
      }
    }
  </style>
  <div class="overlay">
    <div class="content">
      <slot></slot>
      <div class="close-icon">
        <game-icon icon="close"></game-icon>
      </div>
    </div>
  </div>
`

export class GameModal extends HTMLElement {
  constructor() {
    super()
    this.attachShadow({ mode: 'open' })
  }

  connectedCallback() {
    this.shadowRoot.appendChild(modalTemplate.content.cloneNode(true))
    this.addEventListener('click', (e) => {
      this.shadowRoot
        .querySelector('.content')
        .classList.add('closing')
    })
    this.shadowRoot.addEventListener('animationend', (e) => {
      if (e.animationName !== 'SlideOut') return
      this.shadowRoot
        .querySelector('.content')
        .classList.remove('closing')
      // this.shadowRoot.querySelector('.content').innerHTML = '';
      this.removeChild(this.firstChild)
      this.removeAttribute('open')
    })
  }
}

customElements.define('game-modal', GameModal)
