import { MODAL_ZINDEX } from './game-app';

const navModalTemplate = document.createElement('template');
navModalTemplate.innerHTML = `
  <style>
    .overlay-nav {
      display: none;
      position: absolute;
      width: 100%;
      height: 100%;
      top: 0;
      left: 0;
      z-index: ${MODAL_ZINDEX};
      background-color: transparent;
      justify-content: left;
      align-items: unset;
    }

    :host([open]) .overlay-nav {
      display: flex;
    }

    .content-nav {
      position: relative;
      border: 1px solid var(--color-tone-6);
      background-color: var(--modal-content-bg);
      color: var(--color-tone-1);
      overflow-y: auto;
      animation: SlideRight 200ms;
      max-width: var(--game-max-width);
      box-sizing: border-box;
      width: 100%;
      border-radius: 0px;
      box-shadow: 3px 5px 5px rgba(0, 0, 0, 0.15);
      max-height: calc(100% - var(--header-height) - 1px);
      margin-top: calc(var(--header-height) + 1px);
      padding: 0px;
    }

    @media (min-width: 415px) {
      .content-nav {
        width: 375px;
      }
    }

    .content-nav.closing-nav {
      animation: SlideLeft 200ms;
    }

    .close-icon-nav {
      width: 24px;
      height: 24px;
      position: absolute;
      top: 16px;
      right: 16px;
    }

    game-icon {
      position: fixed;
      user-select: none;
      cursor: pointer;
    }

    @keyframes SlideRight {
      0% {
        transform: translateX(-100px);
        opacity: 0;
      }
      100% {
        transform: translateX(0px);
        opacity: 1;
      }
    }
    @keyframes SlideLeft {
      0% {
        transform: translateX(0px);
        opacity: 1;
      }
      90% {
        opacity: 0;
      }
      100% {
        opacity: 0;
        transform: translateX(-200px);
      }
    }
  </style>
  <div class="overlay-nav">
    <div class="content-nav">
      <slot></slot>
      <div class="close-icon-nav">
        <game-icon icon="close"></game-icon>
      </div>
    </div>
  </div>
`;

export class NavModal extends HTMLElement {
  constructor() {
    super();
    this.attachShadow({ mode: 'open' });
  }

  connectedCallback() {
    this.shadowRoot.appendChild(navModalTemplate.content.cloneNode(true));
    this.addEventListener('click', (e) => {
      this.shadowRoot.querySelector('.content-nav').classList.add('closing-nav');
    });
    this.shadowRoot.addEventListener('animationend', (e) => {
      if (e.animationName !== 'SlideLeft') return;
      this.shadowRoot.querySelector('.content-nav').classList.remove('closing-nav');
      // this.shadowRoot.querySelector('.content').innerHTML = '';
      this.removeChild(this.firstChild);
      this.removeAttribute('open');
    });
  }
}

customElements.define('game-nav-modal', NavModal);
