import { PAGE_ZINDEX } from './game-app'

const pageTemplate = document.createElement('template')
pageTemplate.innerHTML = `
  <style>
    .overlay {
      display: none;
      position: absolute;
      width: 100%;
      height: 100%;
      top: 0;
      left: 0;
      justify-content: center;
      background-color: var(--color-background);
      animation: SlideIn 100ms linear;
      z-index: ${PAGE_ZINDEX};
    }

    :host([open]) .overlay {
      display: flex;
    }

    .content {
      position: relative;
      color: var(--color-tone-1);
      padding: 0 32px;
      max-width: var(--game-max-width);
      width: 100%;
      overflow-y: auto;
      height: 100%;
      display: flex;
      flex-direction: column;
    }

    .content-container {
      height: 100%;
    }

    .overlay.closing {
      animation: SlideOut 150ms linear;
    }

    header {
      display: flex;
      justify-content: center;
      align-items: center;
      position: relative;
    }

    h1 {
      font-weight: 700;
      font-size: 16px;
      letter-spacing: 0.5px;
      text-transform: uppercase;
      text-align: center;
      margin-bottom: 10px;
    }

    game-icon {
      position: absolute;
      right: 0;
      user-select: none;
      cursor: pointer;
    }

    @media only screen and (min-device-width : 320px) and (max-device-width : 480px) {
      .content {
        max-width: 100%;
        padding: 0;
      }
      game-icon {
        padding: 0 16px;
      }
    }

    @keyframes SlideIn {
      0% {
        transform: translateY(30px);
        opacity: 0;
      }
      100% {
        transform: translateY(0px);
        opacity: 1;
      }
    }
    @keyframes SlideOut {
      0% {
        transform: translateY(0px);
        opacity: 1;
      }
      90% {
        opacity: 0;
      }
      100% {
        opacity: 0;
        transform: translateY(60px);
      }
    }
  </style>
  <div class="overlay">
    <div class="content">
      <header>
        <h1><slot></slot></h1>
        <game-icon icon="close"></game-icon>
      </header>
      <div class="content-container">
        <slot name="content"></slot>
      </div>
    </div>
  </div>
`

export class GamePage extends HTMLElement {
  constructor() {
    super()
    this.attachShadow({ mode: 'open' })
  }

  connectedCallback() {
    this.shadowRoot.appendChild(pageTemplate.content.cloneNode(true))
    this.shadowRoot
      .querySelector('game-icon')
      .addEventListener('click', (e) => {
        this.shadowRoot
          .querySelector('.overlay')
          .classList.add('closing')
      })
    this.shadowRoot.addEventListener('animationend', (e) => {
      if (e.animationName !== 'SlideOut') return
      this.shadowRoot
        .querySelector('.overlay')
        .classList.remove('closing')
      // this.shadowRoot.querySelector('.content').innerHTML = '';
      Array.from(this.childNodes).forEach((child) => {
        this.removeChild(child)
      })
      // this.removeChild(this.firstChild);
      this.removeAttribute('open')
    })
  }
}

customElements.define('game-page', GamePage)
