/**
 * This code is used to help ensure users download the latest Wordle bundle. 
 */

const REFRESH_KEY = 'nyt-wordle-refresh'
const ls = window.localStorage
const REFRESH_INTERVAL = 43200 * 1000 // 12 hours

let timer;

/**
 * Refreshes the page if you're on the nyties domain. Loads the nyt
 * url if you are not. 
 */
function refreshPage() {
  const nytRegex = /.*\.nytimes\.com/g
  const href = window.location.href;

  if (!href.match(nytRegex)) {
    window.location.replace('https://www.nytimes.com/games/wordle');
  } else {
    window.location.reload(true);
  }
}

/**
 * Use this function to make the user's page refresh by setting a local storage value. An interval 
 * runs every 12 hours to check if the value exists and, if found, will refresh the page
 */
export function setRefreshFlag() {
  ls.setItem(REFRESH_KEY, true);
}

/**
 * Removes nyt-wordle-refresh flag
 */
export function removeRefreshFlag() {
  ls.removeItem(REFRESH_KEY);
}

/**
 * Checks if the game should refresh by reading the flag value
 */
function shouldRefresh() {
  return !!ls.getItem(REFRESH_KEY)
}

/**
 * Kicks off the refresh timer and returns the interval reference
 */
export function startRefreshTimer() {
  if (timer) return timer

  timer = setInterval(() => {
    if (shouldRefresh()) {
      removeRefreshFlag();
      refreshPage();
    }
  }, REFRESH_INTERVAL);

  return timer
}