import { getStats } from './statistics'
import { getShareMessage, share } from './share'
import { WIN, IN_PROGRESS } from './game-app'

const MIN_BAR_PERC = 7

const gameStatsTemplate = document.createElement('template')

gameStatsTemplate.innerHTML = `
  <style>
    .container {
      display: flex;
      flex-direction: column;
      align-items: center;
      justify-content: center;
      padding-top: 16px;
    }
    h1 {
      font-weight: 700;
      font-size: 16px;
      letter-spacing: 0.5px;
      text-transform: uppercase;
      text-align: center;
      margin-bottom: 10px;
    }
  
    #statistics {
      display: flex;
      margin-bottom: 
    }

    .statistic-container {
      flex: 1;
    }

    .statistic-container .statistic {
      font-size: 36px;
      font-weight: 400;
      display: flex;
      align-items: center;
      justify-content: center;
      text-align: center;
      letter-spacing: 0.05em;
      font-variant-numeric: proportional-nums;
    }

    .statistic.timer {
      font-variant-numeric: initial;
    }

    .statistic-container .label {
      font-size: 12px;
      display: flex;
      align-items: center;
      justify-content: center;
      text-align: center;
    }

    #guess-distribution {
      width: 80%;
    }

    .graph-container {
      width: 100%;
      height: 20px;
      display: flex;
      align-items: center;
      padding-bottom: 4px;
      font-size: 14px;
      line-height: 20px;
    }

    .graph-container .graph {
      width: 100%;
      height: 100%;
      padding-left: 4px;
    }

    .graph-container .graph .graph-bar {
      height: 100%;
      /* Assume no wins */
      width: 0%;
      position: relative;
      background-color: var(--color-absent);
      display: flex;
      justify-content: center;
    }

    .graph-container .graph .graph-bar.highlight {
      background-color: var(--color-correct);
    }

    .graph-container .graph .graph-bar.align-right {
      justify-content: flex-end;
      padding-right: 8px;
    }

    .graph-container .graph .num-guesses {
      font-weight: bold;
      color: var(--tile-text-color);
    }

    #statistics,
    #guess-distribution {
      padding-bottom: 10px;
    }

    .footer {
      display: flex;
      width: 100%;
    }

    #promo {
      margin-top: 12px;
      width: 100%;
    }

    .promo-link {
      display: flex;
      width: 100%;
      align-items: center;
      justify-content: flex-start;
      text-decoration: none;
      padding-top: 16px;
    }

    .promo-text {
      font-family: nyt-franklin;
      font-size: 14px;
      line-height: 16px;
      color: var(--color-tone-1);
      flex-grow: 2;
    }

    @media (max-width: 500px) {
      .promo-text-primary {
        display: block;
      }

      .promo-text-secondary {
        display: block;
      }
    }

    .promo-cta {
      display: block;
      font-family: nyt-franklin-700;
      margin-top: 4px;
    }

    .promo-link:hover .promo-cta {
      text-decoration: underline;
    }

    .promo-icon {
      width: var(--promo-icon-width);
      height: var(--promo-icon-height);
      margin-right: 16px;
      border-radius: var(--promo-icon-border-radius);
      padding: var(--promo-icon-padding);
    }

    .promo-arrow {
      justify-self: flex-end;
    }

    .promo-link:hover {
      --svg-arrow-fill: var(--svg-arrow-fill-hover);
      --svg-arrow-stroke: var(--svg-arrow-stroke-hover);
    } 

    .rule {
      margin-left: -16px;
      height: 1px;
      position: absolute;
      width: 100%;
      background-color: var(--color-tone-4);
    }

    .countdown {
      border-right: 1px solid var(--color-tone-1);
      padding-right: 12px;
      width: 50%;
    }

    .share {
      display: flex;
      justify-content: center;
      align-items: center;
      padding-left: 12px;
      width: 50%;
    }

    .no-data {
      text-align: center;
    }

    button#share-button {
      background-color: var(--key-bg-correct);
      color: var(--key-evaluated-text-color);
      font-family: inherit;
      font-weight: bold;
      border-radius: 4px;
      cursor: pointer;
      border: none;
      user-select: none;
      display: flex;
      justify-content: center;
      align-items: center;
      text-transform: uppercase;
      -webkit-tap-highlight-color: rgba(0,0,0,0.3);
      width: 80%;
      font-size: 20px;
      height: 52px;
      -webkit-filter: brightness(100%);
    }
    button#share-button:hover {
      opacity: 0.9;
    }
    button#share-button game-icon {
      width: 24px;
      height: 24px;
      padding-left: 8px;
    }
  </style>

  <div class="container">
    <h1>Statistics</h1>
    <div id="statistics"></div>
    <h1>Guess Distribution</h1>
    <div id="guess-distribution"></div>
    <div class="footer"></div>
    <div id="promo"></div>
  </div>
`;
// TODO: stop width shifting for time
// TODO: better copy
// TODO: bit weird we're using statistic for timer

const statisticContainerTemplate = document.createElement('template')
statisticContainerTemplate.innerHTML = `
  <div class="statistic-container">
    <div class="statistic"></div>
    <div class="label"></div>
  </div>
`

const graphContainerTemplate = document.createElement('template')
graphContainerTemplate.innerHTML = `
    <div class="graph-container">
      <div class="guess"></div>
      <div class="graph">
        <div class="graph-bar">
          <div class="num-guesses">
        </div>
      </div>
      </div>
    </div>
`

const footerContentTemplate = document.createElement('template')
footerContentTemplate.innerHTML = `
  <div class="countdown">
    <h1>Next WORDLE</h1>
    <div id="timer">
      <div class="statistic-container">
        <div class="statistic timer">
          <countdown-timer></countdown-timer>
        </div>
      </div>
    </div>
  </div>
  <div class="share">
    <button id="share-button">
      Share <game-icon icon="share"></game-icon>
    </button>
  </div>
`

const promoContentTemplate = document.createElement('template')
promoContentTemplate.innerHTML = `
  <div class="rule"></div>
  <a href="https://www.nytimes.com/puzzles/spelling-bee" class="promo-link">
    <div class="promo-icon">
      <svg width="42" height="44" viewBox="0 0 42 44" fill="none" xmlns="http://www.w3.org/2000/svg">
        <path d="M24.9528 14.8924L28.952 21.8171L24.9528 28.7417H16.9628L12.9636 21.8171L16.9628 14.8924H24.9528Z" fill="#F7DA21" stroke="#F7DA21" stroke-width="0.495011"/>
        <path d="M12.9892 21.7667L16.9884 28.6914L12.9892 35.616H4.99921L1 28.6914L4.99921 21.7667H12.9892Z" fill="white" stroke="#121212" stroke-width="1.5"/>
        <path d="M17.0056 14.8493L13.0064 7.92463L17.0056 1L24.9955 1L28.9948 7.92463L24.9955 14.8493L17.0056 14.8493Z" fill="white" stroke="#121212" stroke-width="1.5"/>
        <path d="M37.0008 7.96736L41 14.892L37.0008 21.8166H29.0108L25.0116 14.892L29.0108 7.96736H37.0008Z" fill="white" stroke="#121212" stroke-width="1.5"/>
        <path d="M12.9972 7.96736L16.9964 14.892L12.9972 21.8166H5.00727L1.00806 14.892L5.00727 7.96736H12.9972Z" fill="white" stroke="#121212" stroke-width="1.5"/>
        <path d="M37.0008 21.8089L41 28.7335L37.0008 35.6582H29.0108L25.0116 28.7335L29.0108 21.8089H37.0008Z" fill="white" stroke="#121212" stroke-width="1.5"/>
        <path d="M24.9947 28.7333L28.9939 35.658L24.9947 42.5826H17.0047L13.0055 35.658L17.0047 28.7333H24.9947Z" fill="white" stroke="#121212" stroke-width="1.5"/>
      </svg>
    </div>
    <div class="promo-text">
      <span class="promo-text-primary">How many words can you find </span>
      <span class="promo=text-secondary">using 7 letters?</span>
      <span class="promo-cta">Play Spelling Bee</span>
    </div>
    <div class="promo-arrow">
      <svg width="24" height="24" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
        <circle cx="12" cy="12" r="12" fill="var(--svg-arrow-fill)"/>
        <path d="M10.4038 6L15.8076 11.4038L10.4038 16.8076" stroke="var(--svg-arrow-stroke)" stroke-width="1.5" stroke-linecap="round"/>
      </svg>

    </div>
  </a>
`;

const statNameToLabelMap = {
  currentStreak: 'Current Streak',
  maxStreak: 'Max Streak',
  winPercentage: 'Win %',
  gamesPlayed: 'Played',
  gamesWon: 'Won',
  averageGuesses: 'Av. Guesses',
}
export class GameStats extends HTMLElement {
  stats = {}
  gameApp

  constructor() {
    super()
    this.attachShadow({ mode: 'open' })
    this.stats = getStats()
  }

  connectedCallback() {
    // console.log('game-stats connected callback');
    this.shadowRoot.appendChild(
      gameStatsTemplate.content.cloneNode(true),
    )
    const $statistics = this.shadowRoot.getElementById('statistics')
    const $guessDistribution = this.shadowRoot.getElementById(
      'guess-distribution',
    )
    const $promoContainer = this.shadowRoot.getElementById('promo')
    const maxGuesses = Math.max(...Object.values(this.stats.guesses))

    $promoContainer.appendChild(promoContentTemplate.content.cloneNode(true))
    $promoContainer.addEventListener('click', (e) => {
        e.stopPropagation()
    })

    if (
      Object.values(this.stats.guesses).every((guess) => guess === 0)
    ) {
      const $noData = document.createElement('div')
      $noData.classList.add('no-data')
      $noData.innerText = 'No Data'
      $guessDistribution.appendChild($noData)
    } else {
      for (
        let i = 1;
        i < Object.keys(this.stats.guesses).length;
        i++
      ) {
        const guess = i
        const numGuesses = this.stats.guesses[i]
        const $graphContainer =
          graphContainerTemplate.content.cloneNode(true)

        const percentGuesses = Math.max(
          MIN_BAR_PERC,
          Math.round((numGuesses / maxGuesses) * 100),
        )

        // console.log({ guess, numGuesses, percentGuesses });
        $graphContainer.querySelector('.guess').textContent = guess
        const $graphBar = $graphContainer.querySelector('.graph-bar')
        $graphBar.style.width = `${percentGuesses}%`
        if (typeof numGuesses === 'number') {
          $graphContainer.querySelector('.num-guesses').textContent =
            numGuesses
          if (numGuesses > 0) {
            $graphBar.classList.add('align-right')
          }
          const highlightGuess = parseInt(
            this.getAttribute('highlight-guess'),
            10,
          )
          if (highlightGuess && i === highlightGuess) {
            $graphBar.classList.add('highlight')
          }
        }
        $guessDistribution.appendChild($graphContainer)
      }
    }

    ;[
      'gamesPlayed',
      'winPercentage',
      'currentStreak',
      'maxStreak',
    ].forEach((statName) => {
      const label = statNameToLabelMap[statName]
      const stat = this.stats[statName]
      const $statisticContainer =
        statisticContainerTemplate.content.cloneNode(true)
      $statisticContainer.querySelector('.label').textContent = label
      $statisticContainer.querySelector('.statistic').textContent =
        stat
      $statistics.appendChild($statisticContainer)
    })

    if (this.gameApp.gameStatus !== IN_PROGRESS) {
      // Only show the countdown/share button if game is won or lost
      const $footer = this.shadowRoot.querySelector('.footer')
      const $footerContent =
        footerContentTemplate.content.cloneNode(true)
      $footer.appendChild($footerContent)

      // TODO: Only append the share button if the browser supports some form of sharing.

      this.shadowRoot
        .querySelector('button#share-button')
        .addEventListener('click', (e) => {
          e.preventDefault()
          e.stopPropagation()
          // console.log(this.gameApp.dayOffset);
          const message = getShareMessage({
            evaluations: this.gameApp.evaluations,
            dayOffset: this.gameApp.dayOffset,
            rowIndex: this.gameApp.rowIndex,
            isHardMode: this.gameApp.hardMode,
            isWin: this.gameApp.gameStatus === WIN,
          })

          const onCopy = () => {
            this.gameApp.addToast(
              'Copied results to clipboard',
              2000,
              true,
            )
          }

          const onFail = () => {
            this.gameApp.addToast('Share failed', 2000, true)
          }

          share(message, onCopy, onFail)
        })
    }
  }
}

customElements.define('game-stats', GameStats)
