const gameSwitchTemplate = document.createElement('template')
gameSwitchTemplate.innerHTML = `
  <style>
    :host {
    }
    .container {
      display: flex;
      justify-content: space-between;
    }
    .switch {
      height: 20px;
      width: 32px;
      vertical-align: middle;
      /* not quite right */
      background: var(--color-tone-3);
      border-radius: 999px;
      display: block;
      position: relative;
    }
    .knob {
      display: block;
      position: absolute;
      left: 2px;
      top: 2px;
      height: calc(100% - 4px);
      width: 50%;
      border-radius: 8px;
      background: var(--white);
      transform: translateX(0);
      transition: transform 0.3s;
    }
    :host([checked]) .switch {
      background: var(--color-correct);
    }
    :host([checked]) .knob {
      transform: translateX(calc(100% - 4px));
    }
    :host([disabled]) .switch {
      opacity: 0.5;
    }
  </style>
  <div class="container">
    <label><slot></slot></label>
    <div class="switch">
      <span class="knob"></div>
    </div>
  </div>
`

export class GameSwitch extends HTMLElement {
  constructor() {
    super()
    this.attachShadow({ mode: 'open' })
  }

  static get observedAttributes() {
    return ['checked']
  }

  connectedCallback() {
    this.shadowRoot.appendChild(
      gameSwitchTemplate.content.cloneNode(true),
    )
    this.shadowRoot
      .querySelector('.container')
      .addEventListener('click', (e) => {
        e.stopPropagation()
        const isChecked = this.hasAttribute('checked')

        if (!isChecked) {
          this.setAttribute('checked', '')
        } else {
          this.removeAttribute('checked')
        }

        this.dispatchEvent(
          new CustomEvent('game-switch-change', {
            bubbles: true,
            composed: true,
            detail: {
              name: this.getAttribute('name'),
              checked: this.hasAttribute('checked'),
              disabled: this.hasAttribute('disabled'),
            },
          }),
        )
      })
  }
}

customElements.define('game-switch', GameSwitch)
