const gameThemeManagerTemplate = document.createElement('template')

gameThemeManagerTemplate.innerHTML = `
  <slot></slot>
`

export const DARK_THEME_KEY = 'nyt-wordle-darkmode';
export const COLORBLIND_THEME_KEY = 'nyt-wordle-cbmode';
export class GameThemeManager extends HTMLElement {
  isDarkTheme = false
  isColorBlindTheme = false

  constructor() {
    super()
    this.attachShadow({ mode: 'open' })

    const localStorageDarkTheme = JSON.parse(
      window.localStorage.getItem(DARK_THEME_KEY),
    )
    const systemDarkTheme = window.matchMedia(
      '(prefers-color-scheme: dark)',
    ).matches
    const localStorageColorBlindTheme = JSON.parse(
      window.localStorage.getItem(COLORBLIND_THEME_KEY),
    )

    if (
      localStorageDarkTheme === true ||
      localStorageDarkTheme === false
    ) {
      this.setDarkTheme(localStorageDarkTheme)
    } else if (systemDarkTheme) {
      this.setDarkTheme(true)
    }
    if (
      localStorageColorBlindTheme === true ||
      localStorageColorBlindTheme === false
    ) {
      this.setColorBlindTheme(localStorageColorBlindTheme)
    }

    // Expose instance on window to allow migration of theme data
    window.themeManager = this;
  }

  setDarkTheme(enabled) {
    const body = document.querySelector('body')
    if (enabled && !body.classList.contains('nightmode')) {
      body.classList.add('nightmode')
    } else {
      body.classList.remove('nightmode')
    }
    this.isDarkTheme = enabled
    window.localStorage.setItem(
      DARK_THEME_KEY,
      JSON.stringify(enabled),
    )
  }

  setColorBlindTheme(enabled) {
    const body = document.querySelector('body')
    if (enabled && !body.classList.contains('colorblind')) {
      body.classList.add('colorblind')
    } else {
      body.classList.remove('colorblind')
    }
    this.isColorBlindTheme = enabled
    window.localStorage.setItem(
      COLORBLIND_THEME_KEY,
      JSON.stringify(enabled),
    )
  }

  connectedCallback() {
    this.shadowRoot.appendChild(
      gameThemeManagerTemplate.content.cloneNode(true),
    )

    this.shadowRoot.addEventListener('game-setting-change', (e) => {
      const { name, checked } = e.detail
      switch (name) {
        case 'dark-theme': {
          this.setDarkTheme(checked)
          return
        }
        case 'color-blind-theme': {
          this.setColorBlindTheme(checked)
          return
        }
      }
    })
  }
}

customElements.define('game-theme-manager', GameThemeManager)
