import merge from 'lodash-es/merge'

import { captureSentryException  } from './utils/sentry';

// Helpers for reading/writing game state to local storage
const KEY = 'nyt-wordle-state';

const initialState = {
  boardState: null,
  evaluations: null,
  rowIndex: null,
  solution: null,
  gameStatus: null,
  lastPlayedTs: null,
  lastCompletedTs: null,
  restoringFromLocalStorage: null,
  hardMode: false,
}

function setGameState(gameState) {
  try {
    window.localStorage.setItem(KEY, JSON.stringify(gameState))
  } catch (error) {
    captureSentryException(error)
  }
}

export function getGameState() {
  // Hacky method of deepcloning initial gamestate if none exist in local storage
  try {
    const gameState = window.localStorage.getItem(KEY) || JSON.stringify(initialState)
    return JSON.parse(gameState)
  } catch (error) {
    captureSentryException(error)
    return { ...initialState }
  }
}

export function updateLastCompletedTimestamp(timestamp) {
  const gameState = getGameState()
  gameState.lastCompletedTimestamp = timestamp
  setGameState(gameState)
}

export function updateGameState(updatedGameState) {
  const gameState = getGameState()
  const newGameState = merge(gameState, updatedGameState)
  setGameState(newGameState)
}

/**
 * In case of roll-back of phoenix-wordle, we want to clear its local storage.
 * So that once we roll-forward, the legacy state serves as the most up to date source of truth.
 */
 export function clearMoogleState() {
  const moogleKey = 'nyt-wordle-moogle/ANON'
  try {
    window.localStorage.removeItem(moogleKey)
  } catch (e) {
    captureSentryException(e)
  }
}
