import {
  DARK_THEME_KEY,
  COLORBLIND_THEME_KEY,
} from './game-theme-manager'
import { CORRECT, PRESENT, ABSENT } from './gamelogic'
import { NUM_ROWS } from './game-app'
import { isMobile, isFirefox } from './utils/deviceIdentifiers'
import * as clipboard from 'clipboard-polyfill/text'

function getAbsent(isDarkTheme) {
  return isDarkTheme ? '⬛' : '⬜'
}

function getCorrect(isColorBlindTheme) {
  return !isColorBlindTheme ? '🟩' : '🟧'
}

function getPresent(isColorBlindTheme) {
  return !isColorBlindTheme ? '🟨' : '🟦'
}

export function getShareMessage({
  evaluations,
  dayOffset,
  rowIndex,
  isHardMode,
  isWin,
}) {
  // console.log({ evaluations, dayOffset, rowIndex });

  // TODO: ohboy, these should not be here
  const isDarkTheme = JSON.parse(
    window.localStorage.getItem(DARK_THEME_KEY),
  )
  const isColorBlindTheme = JSON.parse(
    window.localStorage.getItem(COLORBLIND_THEME_KEY),
  )

  let title = `Wordle ${dayOffset}`

  title += ` ${isWin ? rowIndex : 'X'}/${NUM_ROWS}`

  if (isHardMode) {
    title += '*'
  }

  // message += '\n\n';
  let grid = ''

  evaluations.forEach((row) => {
    if (!row) return
    row.forEach((evaluation) => {
      if (!evaluation) return
      let evaluationString = ''
      switch (evaluation) {
        case CORRECT: {
          evaluationString = getCorrect(isColorBlindTheme)
          break
        }
        case PRESENT: {
          evaluationString = getPresent(isColorBlindTheme)
          break
        }
        case ABSENT: {
          evaluationString = getAbsent(isDarkTheme)
          break
        }
        default:
          break
      }
      grid += evaluationString
    })
    grid += '\n'
  })

  // trim the grid to remove the last whitespace that was added
  const message = `${title}\n\n${grid.trimEnd()}`

  return {
    text: message,
  }
}

export function share(message, onCopy, onFail) {
  try {
    if (
      isMobile() &&
      !isFirefox() &&
      navigator.share !== undefined &&
      !!navigator.canShare &&
      navigator.canShare(message)
    ) {
      navigator.share(message)
    } else {
      clipboard.writeText(message.text).then(onCopy, onFail)
    }
  } catch (e) {
    onFail()
    console.error('Share failed', e)
  }
}
