/*
 Stats are only ever read and set via local storage. Use the helpers if you want
 to update them.
*/

/**
 * originally 'statistics' but for the NYT migration, changing to something
 * less generic to avoid a potential collision
 */
import { captureSentryException } from "./utils/sentry"

const STATS_KEY = 'nyt-wordle-statistics'
const FAIL = 'fail'

const initialStats = {
  currentStreak: 0,
  maxStreak: 0,
  guesses: {
    1: 0,
    2: 0,
    3: 0,
    4: 0,
    5: 0,
    6: 0,
    [FAIL]: 0,
  },
  winPercentage: 0,
  gamesPlayed: 0,
  gamesWon: 0,
  averageGuesses: 0,
}

function setStats(stats) {
  try {
    window.localStorage.setItem(STATS_KEY, JSON.stringify(stats))
  } catch (error) {
    captureSentryException(error)
  }
}

export function getStats() {
  // Hacky method of deepcloning initial stats if none exist in local storage
  try {
    const stats = window.localStorage.getItem(STATS_KEY) || JSON.stringify(initialStats)
    return JSON.parse(stats)
  } catch (error) {
    captureSentryException(error)
  }
}

export function updateStats({ isWin, isStreak, numGuesses }) {
  const stats = getStats()

  if (isWin) {
    stats.guesses[numGuesses] += 1
    if (isStreak) {
      stats.currentStreak += 1
    } else {
      stats.currentStreak = 1
    }
  } else {
    stats.currentStreak = 0
    stats.guesses[FAIL] += 1
  }
  stats.maxStreak = Math.max(stats.currentStreak, stats.maxStreak)

  stats.gamesPlayed += 1
  stats.gamesWon += isWin ? 1 : 0
  stats.winPercentage = Math.round(
    (stats.gamesWon / stats.gamesPlayed) * 100,
  )
  stats.averageGuesses = Math.round(
    Object.entries(stats.guesses).reduce((acc, [key, val]) => {
      return key !== FAIL ? (acc += key * val) : acc
    }, 0) / stats.gamesWon,
  )

  setStats(stats)
}
