import * as Sentry from '@sentry/browser';
import { BrowserTracing } from "@sentry/tracing";

const sentryConfig = {
  dsn: 'https://412ad7ade69c4fd59ced026a41c8a644@o82024.ingest.sentry.io/6236168', // The DSN/public key for games-wordle project in Sentry
  maxBreadcrumbs: 30,
  integrations: [new BrowserTracing()],
  release: window.wordle?.hash || "",
  tracesSampleRate: 0.005, // transaction send rate of 0.5%
  sampleRate: 0.005, // error send rate of 0.5%
  denyUrls: [ // from games-phoenix
    // Chrome extensions
    /extensions\//i,
    /^chrome:\/\//i,
    /safari-extension/,
    /safari-web-extension/,
    /moz-extension/,
    /chrome-extension/,
    /data:application/,
    // Users downloading files
    /file:\/\/\//,
    // Other plugins
    /127\.0\.0\.1:4001\/isrunning/i, // Cacaoweb
    /webappstoolbarba\.texthelp\.com\//i,
    /metrics\.itunes\.apple\.com\.edgesuite\.net\//i,
    /translate\.goog/,
    // Ad networks and partners
    /googletagservices/,
    /moatads/,
    /adtech/,
    /contextual\.media\.net/,
    /platform\.iteratehq\.com/,
    /chartbeat/,
    /flashtalking/,
    /doubleverify/,
    /google-analytics/,
    /3lift/,
    /celtra/,
    // Proxies
    /4everproxy/,
    /hideproxy/,
    /proxysite/,
    /us-proxy/,
    /geoedge/,
    /grumi-ip/,
  ]
}

export const initSentry = () => {
  try {
    Sentry.init(sentryConfig)
    console.log('Initializing Sentry...')
  } catch (error) {
    console.error('Error initializing Sentry', error)
  }
}

export const captureSentryMessage = (msg) => {
  Sentry.captureMessage(msg)
}

export const captureSentryException = (error) => {
  Sentry.captureException(error)
}